@@ -1,5 +1,5 @@
 # These functions are
-# Copyright (C) 1998-2015 T.W. Yee, University of Auckland.
+# Copyright (C) 1998-2016 T.W. Yee, University of Auckland.
 # All rights reserved.
 
 
@@ -59,11 +59,11 @@ cao  <- function(formula,
   attr(x, "assign") <- attrassigndefault(x, mt)
   offset <- model.offset(mf)
   if (is.null(offset)) 
-    offset <- 0 # yyy ???
+    offset <- 0  # yyy ???
   w <- model.weights(mf)
-  if (!length(w))
-    w <- rep(1, nrow(mf))
-  else if (ncol(as.matrix(w)) == 1 && any(w < 0))
+  if (!length(w)) {
+    w <- rep_len(1, nrow(mf))
+  } else if (ncol(as.matrix(w)) == 1 && any(w < 0))
     stop("negative weights not allowed")
 
   if (is.character(family))
@@ -83,7 +83,7 @@ cao  <- function(formula,
   cao.fitter <- get(method)
 
 
-  deviance.Bestof <- rep(NA_real_, len = control$Bestof)
+  deviance.Bestof <- rep_len(NA_real_, control$Bestof)
   for (tries in 1:control$Bestof) {
     if (control$trace && (control$Bestof > 1)) {
       cat(paste("\n========================= Fitting model",
